"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("./util");
const ssh_util_1 = require("./ssh-util");
function pemPrivateKey(key) {
    return `-----BEGIN PRIVATE KEY-----\n${util_1.wrapString(key, 64)}-----END PRIVATE KEY-----`;
}
function stripPemFormatting(str) {
    return str
        .replace(/^-----BEGIN (?:RSA )?(?:PRIVATE|PUBLIC) KEY-----$/m, "")
        .replace(/^-----END (?:RSA )?(?:PRIVATE|PUBLIC) KEY-----$/m, "")
        .replace(/[\n\r]/g, "");
}
const prefix = [
    0x30,
    0x82,
    0x04,
    0xbc,
    0x02,
    0x01,
    0x00,
    0x30,
    0x0d,
    0x06,
    0x09,
    0x2a,
    0x86,
    0x48,
    0x86,
    0xf7,
    0x0d,
    0x01,
    0x01,
    0x01,
    0x05,
    0x00,
    0x04,
    0x82,
    0x04,
    0xa6,
];
function pkcs1To8(privateKeyPkcs1Pem) {
    const pem = stripPemFormatting(privateKeyPkcs1Pem);
    const privateKeyPkcs1Array = ssh_util_1.pemToArray(pem);
    const prefixPkcs8 = prefix.concat(privateKeyPkcs1Array);
    const privateKeyPkcs8Pem = ssh_util_1.arrayToPem(prefixPkcs8);
    const pkcs8Pem = pemPrivateKey(privateKeyPkcs8Pem);
    return pkcs8Pem;
}
exports.pkcs1To8 = pkcs1To8;
//# sourceMappingURL=pkcs1To8.js.map